VERSION 1.0 CLASS
BEGIN
  MultiUse = -1  'True
  Persistable = 0  'NotPersistable
  DataBindingBehavior = 0  'vbNone
  DataSourceBehavior  = 0  'vbNone
  MTSTransactionMode  = 0  'NotAnMTSObject
END
Attribute VB_Name = "clsFolderPicker"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = True
Attribute VB_PredeclaredId = False
Attribute VB_Exposed = False
' Autor: F. Schler (frank@activevb.de)
' Datum: 01/2023

Option Explicit

' ----==== Const ====----
Private Const WindowsStoragePickersFolderPicker As String = "Windows.Storage.Pickers.FolderPicker"
Private Const IID_IFolderPicker As String = "{084f7799-f3fb-400a-99b1-7b4a772fd60d}"
Private Const IID_IStorageItem As String = "{4207a996-ca2f-42f7-bde8-8b10457a7f30}"

' ----==== Enums ====----
Private Enum vtb_Interfaces
    
    ' IFolderPicker
    IFolderPicker_GetViewMode = 6
    IFolderPicker_PutViewMode = 7
    IFolderPicker_GetSettingsIdentifier = 8
    IFolderPicker_PutSettingsIdentifier = 9
    IFolderPicker_GetSuggestedStartLocation = 10
    IFolderPicker_PutSuggestedStartLocation = 11
    IFolderPicker_GetCommitButtonText = 12
    IFolderPicker_PutCommitButtonText = 13
    IFolderPicker_GetFileTypeFilter = 14
    IFolderPicker_PickSingleFolderAsync = 15

    ' IStorageItem
    IStorageItem_GetName = 11
    IStorageItem_GetPath = 12

    ' IVector_HSTRING
    IVector_HSTRING_Append = 13
    
End Enum

' ----==== Variablen ====----
Private m_pIFolderPicker As Long
Private m_FolderName As String
Private m_FolderPath As String

' ----==== Class ====----
Private Sub Class_Initialize()
    If GetActivateInstance(WindowsStoragePickersFolderPicker, _
                           IID_IFolderPicker, _
                           m_pIFolderPicker) Then
    End If
End Sub

Private Sub Class_Terminate()
    m_FolderName = vbNullString
    m_FolderPath = vbNullString
    Call ReleaseIfc(m_pIFolderPicker)
End Sub

' ----==== Properties ====----
Public Property Get FolderName() As String
    FolderName = m_FolderName
End Property

Public Property Get FolderPath() As String
    FolderPath = m_FolderPath
End Property

Public Property Get ViewMode() As PickerViewMode
    Dim Ret As PickerViewMode
    If m_pIFolderPicker <> 0& Then
        Dim value As Long
        If Invoke(m_pIFolderPicker, _
                  IFolderPicker_GetViewMode, _
                  VarPtr(value)) = S_OK Then
            Ret = value
        End If
    End If
    ViewMode = Ret
End Property

Public Property Let ViewMode(ByVal value As PickerViewMode)
    If m_pIFolderPicker <> 0& Then
        If Invoke(m_pIFolderPicker, _
                  IFolderPicker_PutViewMode, _
                  value) = S_OK Then
        End If
    End If
End Property

Public Property Get SettingsIdentifier() As String
    Dim Ret As String
    If m_pIFolderPicker <> 0& Then
        Dim value As Long
        If Invoke(m_pIFolderPicker, _
                  IFolderPicker_GetSettingsIdentifier, _
                  VarPtr(value)) = S_OK Then
            SettingsIdentifier = GetWindowsString(value)
        End If
    End If
    SettingsIdentifier = Ret
End Property

Public Property Let SettingsIdentifier(ByVal value As String)
    If m_pIFolderPicker <> 0& Then
        Dim hString As Long
        hString = CreateWindowsString(value)
        If hString <> 0& Then
            If Invoke(m_pIFolderPicker, _
                      IFolderPicker_PutSettingsIdentifier, _
                      hString) = S_OK Then
            End If
            Call DeleteWindowsString(hString)
        End If
    End If
End Property

Public Property Get SuggestedStartLocation() As PickerLocationId
    Dim Ret As PickerLocationId
    If m_pIFolderPicker <> 0& Then
        Dim value As Long
        If Invoke(m_pIFolderPicker, _
                  IFolderPicker_GetSuggestedStartLocation, _
                  VarPtr(value)) = S_OK Then
            Ret = value
        End If
    End If
    SuggestedStartLocation = Ret
End Property

Public Property Let SuggestedStartLocation(ByVal value As PickerLocationId)
    If m_pIFolderPicker <> 0& Then
        If Invoke(m_pIFolderPicker, _
                  IFolderPicker_PutSuggestedStartLocation, _
                  value) = S_OK Then
        End If
    End If
End Property

Public Property Get CommitButtonText() As String
    Dim Ret As String
    If m_pIFolderPicker <> 0& Then
        Dim value As Long
        If Invoke(m_pIFolderPicker, _
                  IFolderPicker_GetCommitButtonText, _
                  VarPtr(value)) = S_OK Then
            Ret = GetWindowsString(value)
        End If
    End If
    CommitButtonText = Ret
End Property

Public Property Let CommitButtonText(ByVal value As String)
    If m_pIFolderPicker <> 0& Then
        Dim hString As Long
        hString = CreateWindowsString(value)
        If hString <> 0& Then
            If Invoke(m_pIFolderPicker, _
                      IFolderPicker_PutCommitButtonText, _
                      hString) = S_OK Then
            End If
            Call DeleteWindowsString(hString)
        End If
    End If
End Property

' ----==== Functions ====----
Public Function PickSingleFolderAsync(ByVal ownerHwnd As Long) As Boolean
    Dim Ret As Boolean
    m_FolderName = vbNullString
    m_FolderPath = vbNullString
    If m_pIFolderPicker <> 0& Then
        If FileTypeFilterAppend("*") Then
            If InitializeWithWindow(m_pIFolderPicker, ownerHwnd) Then
                Dim pIStorageFolder As Long
                If Invoke(m_pIFolderPicker, _
                          IFolderPicker_PickSingleFolderAsync, _
                          VarPtr(pIStorageFolder)) = S_OK Then
                    If Await(pIStorageFolder) Then
                        Dim pIStorageItem As Long
                        If QueryIfc(pIStorageFolder, _
                                    IID_IStorageItem, _
                                    pIStorageItem) Then
                            Dim value As Long
                            If Invoke(pIStorageItem, _
                                      IStorageItem_GetPath, _
                                      VarPtr(value)) = S_OK Then
                                m_FolderPath = GetWindowsString(value)
                                If Invoke(pIStorageItem, _
                                          IStorageItem_GetName, _
                                          VarPtr(value)) = S_OK Then
                                    m_FolderName = GetWindowsString(value)
                                    Ret = True
                                End If
                            End If
                            Call ReleaseIfc(pIStorageItem)
                        End If
                    End If
                    Call ReleaseIfc(pIStorageFolder)
                End If
            End If
        End If
    End If
    PickSingleFolderAsync = Ret
End Function

Private Function FileTypeFilterAppend(ByVal value As String) As Boolean
    Dim Ret As Boolean
    If m_pIFolderPicker <> 0& Then
        Dim hString As Long
        hString = CreateWindowsString(value)
        If hString <> 0& Then
            Dim pIVector_HSTRING As Long
            If Invoke(m_pIFolderPicker, _
                      IFolderPicker_GetFileTypeFilter, _
                      VarPtr(pIVector_HSTRING)) = S_OK Then
                If Invoke(pIVector_HSTRING, _
                          IVector_HSTRING_Append, _
                          hString) = S_OK Then
                    Ret = True
                End If
                Call ReleaseIfc(pIVector_HSTRING)
            End If
            Call DeleteWindowsString(hString)
        End If
    End If
    FileTypeFilterAppend = Ret
End Function

Private Function Invoke(ByVal pInterface As Long, _
                        ByVal vtb As vtb_Interfaces, _
                        ParamArray var()) As Variant
    If pInterface <> 0& Then
        Invoke = OleInvoke(pInterface, vtb, var)
    End If
End Function
